#include <avr/io.h> 
#include <avr/interrupt.h>   
#include <util/delay.h>
#include <avr/pgmspace.h>

#define uint8		unsigned char
#define int8		signed char
#define uint16		unsigned int
#define int16		signed int
#define uint32		unsigned long
#define int32		signed long

#define LED_1		PD1
#define LED_2		PD2

#define LED1_OFF	PORTD &= ~(1 << LED_1);
#define LED1_ON		PORTD |=  (1 << LED_1);
#define LED2_OFF	PORTD &= ~(1 << LED_2);
#define LED2_ON		PORTD |=  (1 << LED_2);

// all on PORTC
#define LCD_CD		PC7
#define LCD_RW		PC6	// WR in 8080 mode
#define LCD_E		PC5	// RD in 8080 mode
#define LCD_WR		PC6	// WR in 8080 mode
#define LCD_RD		PC5	// RD in 8080 mode
#define LCD_CS		PC3
#define LCD_RES		PC2

#define SD_PORT		PORTB						// use portB to communicate SD
#define SD_CS_PIN	0							// mega128 PortB.0 ss
#define DDR_INI()	DDRB|=_BV(0)|_BV(1)|_BV(2)	// SS, MOSI, SCK for m128 portB, write

#define CLR_CD		PORTC &= ~(1<<LCD_CD);
#define SET_CD		PORTC |=  (1<<LCD_CD);

#define CLR_CS		PORTC &= ~(1<<LCD_CS);
#define SET_CS		PORTC |=  (1<<LCD_CS);

#define CLR_RESET	PORTC &= ~(1<<LCD_RES);
#define SET_RESET	PORTC |=  (1<<LCD_RES);

#define WriteCommand(cmd)	soft_spi_send_byte(0, 0, cmd)
#define WriteData(data)		soft_spi_send_byte(1, 0, data)

extern void SPI_High(void);
extern void SPI_Low(void);

// 6800 mode pin functions
#define CLR_RW		PORTC &= ~(1 << LCD_RW);   // 6800 mode
#define SET_RW		PORTC |=  (1 << LCD_RW);   // 6800 mode
#define CLR_E		PORTC &= ~(1 << LCD_E);    // 6800 mode
#define SET_E		PORTC |=  (1 << LCD_E);    // 6800 mode

// 8080 mode pin functions
#define CLR_WR		PORTC &= ~(1 << LCD_WR);   // 8080 mode
#define SET_WR		PORTC |=  (1 << LCD_WR);   // 8080 mode
#define CLR_RD		PORTC &= ~(1 << LCD_RD);   // 8080 mode
#define SET_RD		PORTC |=  (1 << LCD_RD);   // 8080 mode

const unsigned char crystalfontz_text[100] = { /* 0X02,0X01,0X64,0X00,0X10,0X00, */
0X00,0X00,0X58,0X94,0X94,0X64,0X00,0XFC,0X84,0X84,0XFD,0X00,0X88,0X70,0X88,0X00,
0X00,0X08,0XFC,0X00,0X00,0X78,0X94,0X94,0X64,0X00,0X00,0X00,0X78,0X84,0X84,0X84,
0X78,0X00,0X00,0XFC,0X80,0X80,0X00,0XFC,0X94,0X94,0X94,0X00,0XFC,0X84,0X84,0X78,
0X00,0X00,0X3E,0X41,0X41,0X41,0X22,0X00,0X7C,0X04,0X04,0X04,0X38,0XC0,0X38,0X04,
0X00,0X48,0X54,0X54,0X24,0X04,0X7E,0X44,0X00,0X64,0X54,0X54,0X7C,0X00,0X7F,0X00,
0X04,0X7E,0X05,0X00,0X38,0X44,0X44,0X38,0X00,0X7C,0X04,0X04,0X78,0X04,0X7E,0X44,
0X00,0X64,0X54,0X4C,};


const unsigned char crystalfontz_text_inverted[100] = { /* 0X02,0X01,0X64,0X00,0X10,0X00, */
0XFF,0XFF,0XA7,0X6B,0X6B,0X9B,0XFF,0X03,0X7B,0X7B,0X02,0XFF,0X77,0X8F,0X77,0XFF,
0XFF,0XF7,0X03,0XFF,0XFF,0X87,0X6B,0X6B,0X9B,0XFF,0XFF,0XFF,0X87,0X7B,0X7B,0X7B,
0X87,0XFF,0XFF,0X03,0X7F,0X7F,0XFF,0X03,0X6B,0X6B,0X6B,0XFF,0X03,0X7B,0X7B,0X87,
0XFF,0XFF,0XC1,0XBE,0XBE,0XBE,0XDD,0XFF,0X83,0XFB,0XFB,0XFB,0XC7,0X3F,0XC7,0XFB,
0XFF,0XB7,0XAB,0XAB,0XDB,0XFB,0X81,0XBB,0XFF,0X9B,0XAB,0XAB,0X83,0XFF,0X80,0XFF,
0XFB,0X81,0XFA,0XFF,0XC7,0XBB,0XBB,0XC7,0XFF,0X83,0XFB,0XFB,0X87,0XFB,0X81,0XBB,
0XFF,0X9B,0XAB,0XB3,};

const unsigned char gImage_50x16_logo[100] = { /* 0X12,0X01,0X00,0X32,0X00,0X10, */
0X00,0X00,0X00,0X00,0X0F,0X0F,0X2F,0X6F,0XCF,0X8F,0XCF,0X40,0X1F,0X3F,0X7F,0XC0,
0X8A,0XCA,0X6A,0X2A,0X0A,0X0A,0X0A,0X00,0X00,0X00,0X00,0X0F,0X0F,0X2F,0X6F,0XCF,
0X8F,0XCF,0X40,0X1F,0X3F,0X7F,0XC0,0X8A,0XCA,0X6A,0X2A,0X0A,0X0A,0X0A,0X00,0X00,
0X00,0X00,0X00,0X00,0X00,0X00,0XE0,0XE0,0XE8,0XEC,0XE6,0XE2,0XE6,0X04,0XF0,0XF8,
0XFC,0X06,0XA2,0XA6,0XAC,0XA8,0XA0,0XA0,0XA0,0X00,0X00,0X00,0X00,0XE0,0XE0,0XE8,
0XEC,0XE6,0XE2,0XE6,0X04,0XF0,0XF8,0XFC,0X06,0XA2,0XA6,0XAC,0XA8,0XA0,0XA0,0XA0,
0X00,0X00,0X00,0X00,};

const unsigned char gImage_50x16_logo_inverted[100] = { /* 0X12,0X01,0X00,0X32,0X00,0X10, */
0XFF,0XFF,0XFF,0XFF,0XF0,0XF0,0XD0,0X90,0X30,0X70,0X30,0XBF,0XE0,0XC0,0X80,0X3F,
0X75,0X35,0X95,0XD5,0XF5,0XF5,0XF5,0XFF,0XFF,0XFF,0XFF,0XF0,0XF0,0XD0,0X90,0X30,
0X70,0X30,0XBF,0XE0,0XC0,0X80,0X3F,0X75,0X35,0X95,0XD5,0XF5,0XF5,0XF5,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0X1F,0X1F,0X17,0X13,0X19,0X1D,0X19,0XFB,0X0F,0X07,
0X03,0XF9,0X5D,0X59,0X53,0X57,0X5F,0X5F,0X5F,0XFF,0XFF,0XFF,0XFF,0X1F,0X1F,0X17,
0X13,0X19,0X1D,0X19,0XFB,0X0F,0X07,0X03,0XF9,0X5D,0X59,0X53,0X57,0X5F,0X5F,0X5F,
0XFF,0XFF,0XFF,0XFF,};

//spi low speed
void SPI_Low(void)
{
	SPCR =   _BV(SPE)|_BV(MSTR)|_BV(SPR1)|_BV(SPR0);
	SPSR &= ~_BV(SPI2X);
}

//spi full speed
void SPI_High(void)
{
	SPCR =  _BV(SPE)|_BV(MSTR);
	SPSR |= _BV(SPI2X);
}

//port initialize
void SPI_Init(void)
{
	//DDR_INI();
	SPI_Low();
	SPI_High();
}

void delay(uint32_t twait)
{
	while (twait--)
		asm volatile ("nop");
}

void soft_spi_send_byte(char cmd, char write, char data)
{
	// enable chip_sel
	asm("sbi %0, 7" : : "I" (_SFR_IO_ADDR(PORTB)));	// set chip select
	asm("cbi %0, 7" : : "I" (_SFR_IO_ADDR(PORTB)));	// clear chip select
	
	// Send the data/command flag bit
	asm("cbi %0, 5" : : "I" (_SFR_IO_ADDR(PORTB)));	//	clear the clock
	asm("cbi %0, 6" : : "I" (_SFR_IO_ADDR(PORTB)));	//	clear the SDIN
	asm("sbrc %0, 0" : : "a" (cmd));				//	check if the data should be set or not
	asm("sbi %0, 6" : : "I" (_SFR_IO_ADDR(PORTB)));	//	set the data
	asm("sbi %0, 5" : : "I" (_SFR_IO_ADDR(PORTB)));	//	set the clock
	
	// Send the write flag bit
	asm("cbi %0, 5" : : "I" (_SFR_IO_ADDR(PORTB)));	//	clear the clock
	asm("cbi %0, 6" : : "I" (_SFR_IO_ADDR(PORTB)));	//	clear the SDIN
	asm("sbrc %0, 0" : : "a" (write));				//	check if the data should be set or not
	asm("sbi %0, 6" : : "I" (_SFR_IO_ADDR(PORTB)));	//	set the data
	asm("sbi %0, 5" : : "I" (_SFR_IO_ADDR(PORTB)));	//	set the clock
	
	// Send Bit 7 of data
	asm("cbi %0, 5" : : "I" (_SFR_IO_ADDR(PORTB)));
	asm("cbi %0, 6" : : "I" (_SFR_IO_ADDR(PORTB)));
	asm("sbrc %0, 7" : : "a" (data));
	asm("sbi %0, 6" : : "I" (_SFR_IO_ADDR(PORTB)));
	asm("sbi %0, 5" : : "I" (_SFR_IO_ADDR(PORTB)));
	
	// Send Bit 6 of data
	asm("cbi %0, 5" : : "I" (_SFR_IO_ADDR(PORTB)));
	asm("cbi %0, 6" : : "I" (_SFR_IO_ADDR(PORTB)));
	asm("sbrc %0, 6" : : "a" (data));
	asm("sbi %0, 6" : : "I" (_SFR_IO_ADDR(PORTB)));
	asm("sbi %0, 5" : : "I" (_SFR_IO_ADDR(PORTB)));
	
	// Send bit 5 of data
	asm("cbi %0, 5" : : "I" (_SFR_IO_ADDR(PORTB)));
	asm("cbi %0, 6" : : "I" (_SFR_IO_ADDR(PORTB)));
	asm("sbrc %0, 5" : : "a" (data));
	asm("sbi %0, 6" : : "I" (_SFR_IO_ADDR(PORTB)));
	asm("sbi %0, 5" : : "I" (_SFR_IO_ADDR(PORTB)));
	
	// Send bit 4 of data
	asm("cbi %0, 5" : : "I" (_SFR_IO_ADDR(PORTB)));
	asm("cbi %0, 6" : : "I" (_SFR_IO_ADDR(PORTB)));
	asm("sbrc %0, 4" : : "a" (data));
	asm("sbi %0, 6" : : "I" (_SFR_IO_ADDR(PORTB)));
	asm("sbi %0, 5" : : "I" (_SFR_IO_ADDR(PORTB)));
	
	// Send bit 3 of data
	asm("cbi %0, 5" : : "I" (_SFR_IO_ADDR(PORTB)));
	asm("cbi %0, 6" : : "I" (_SFR_IO_ADDR(PORTB)));
	asm("sbrc %0, 3" : : "a" (data));
	asm("sbi %0, 6" : : "I" (_SFR_IO_ADDR(PORTB)));
	asm("sbi %0, 5" : : "I" (_SFR_IO_ADDR(PORTB)));
	
	// Send bit 2 of data
	asm("cbi %0, 5" : : "I" (_SFR_IO_ADDR(PORTB)));
	asm("cbi %0, 6" : : "I" (_SFR_IO_ADDR(PORTB)));
	asm("sbrc %0, 2" : : "a" (data));
	asm("sbi %0, 6" : : "I" (_SFR_IO_ADDR(PORTB)));
	asm("sbi %0, 5" : : "I" (_SFR_IO_ADDR(PORTB)));
	
	// Send bit 1 of data
	asm("cbi %0, 5" : : "I" (_SFR_IO_ADDR(PORTB)));
	asm("cbi %0, 6" : : "I" (_SFR_IO_ADDR(PORTB)));
	asm("sbrc %0, 1" : : "a" (data));
	asm("sbi %0, 6" : : "I" (_SFR_IO_ADDR(PORTB)));
	asm("sbi %0, 5" : : "I" (_SFR_IO_ADDR(PORTB)));
	
	// Send bit 0 of data
	asm("cbi %0, 5" : : "I" (_SFR_IO_ADDR(PORTB)));
	asm("cbi %0, 6" : : "I" (_SFR_IO_ADDR(PORTB)));
	asm("sbrc %0, 0" : : "a" (data));
	asm("sbi %0, 6" : : "I" (_SFR_IO_ADDR(PORTB)));
	asm("sbi %0, 5" : : "I" (_SFR_IO_ADDR(PORTB)));
	
	// disable chip_sel
	asm("sbi %0, 7" : : "I" (_SFR_IO_ADDR(PORTB)));
	asm("cbi %0, 5" : : "I" (_SFR_IO_ADDR(PORTB)));	// clear the clock
}

void reset_display(void)
{
	CLR_RESET;
	delay(10000);
	SET_RESET;
	delay(10000);
}

void init_OLED()
{
		WriteCommand(0x39);
		_delay_us(200);
		WriteCommand(0x0C);
		_delay_us(200);
		WriteCommand(0x06);
		_delay_us(200);
		WriteCommand(0x1F);
		_delay_us(200);
		WriteCommand(0x02);
		_delay_us(200);
		WriteCommand(0x01);
		_delay_us(200);
}

int main()
{
	DDRB = 0xFF;
	PORTB = 0;

	SPI_High();
	OSCCAL = 0x00;		//normal operation maximum the frequency

	delay(20000L);

	init_OLED();		// initialize OLED module

	while (1)
	{
	WriteCommand(0x80);
	WriteCommand(0x41);
	for(int row=0;row<50;row++)
	{
			WriteData(crystalfontz_text[row]);
	}

	WriteCommand(0x80);
	WriteCommand(0x40);
	for(int row=50;row<100;row++)
	{
			WriteData(crystalfontz_text[row]);
	}

	_delay_ms(2000);

	WriteCommand(0x80);
	WriteCommand(0x41);
	for(int row=0;row<50;row++)
	{
			WriteData(crystalfontz_text_inverted[row]);
	}
	for(int i=0;i<50;i++)
		WriteData(0x00);
	
	WriteCommand(0x80);
	WriteCommand(0x40);
	for(int row=50;row<100;row++)
	{
			WriteData(crystalfontz_text_inverted[row]);
	}
	for(int i=0;i<50;i++)
		WriteData(0x00);

	_delay_ms(2000);
	WriteCommand(0x80);
	WriteCommand(0x41);
	for(int row=0;row<50;row++)
	{
			WriteData(gImage_50x16_logo[row]);
	}

	WriteCommand(0x80);
	WriteCommand(0x40);
	for(int row=50;row<100;row++)
	{
			WriteData(gImage_50x16_logo[row]);
	}

	_delay_ms(2000);
	WriteCommand(0x80);
	WriteCommand(0x41);
	for(int row=0;row<50;row++)
	{
			WriteData(gImage_50x16_logo_inverted[row]);
	}

	WriteCommand(0x80);
	WriteCommand(0x40);
	for(int row=50;row<100;row++)
	{
			WriteData(gImage_50x16_logo_inverted[row]);
	}
	_delay_ms(2000);

	}
	return 0;
}
